/****************************************************************
*
*  Name:          SHAREB
*
*  Function:      share memory/data among multiple processes
*
*  Shows how to:  1. read from and write to shared memory.
*                 2. receive from another process the address of shared data.
*                 3. control access to shared data via mailbox semaphore.
*
*  Written by:    Larry Rush, Quarterdeck Office Systems
*
*  Contact:       Voice:  (213) 392-9851, (213) 392-9701
*                 BBS:    (213) 396-3904, (213) 392-2278
*                 Fax:    (213) 399-3802
*
****************************************************************/

#include <stdio.h>
#include "dvapi.h"

/* minimum API version required */
#define required 0x201

/* arbitrary # times to read/write shared memory */
#define REPS 4

/* API version number */
int version;

/* default object handle */
ulong win;

/* application handle of other process */
ulong apphana;

/* mail-related variables */
int status;
int mallng;

/* read/write loop control variable */
int i;

/* type declarations related to shared data */
/*i* typedef int datatype; *i*/
typedef char datatype[11];
/*r* struct datatype { *r*/
  /*r* struct datatype *link; *r*/
  /*r* int lng; *r*/
  /*r* char data[11]; *r*/
  /*r* }; *r*/

/* type declaration of pointer to shared data */
/*i* typedef datatype *dataptr; *i*/
typedef datatype *dataptr; 
/*r* typedef struct datatype *dataptr; *r*/

/* constant value to be assigned to shared memory */
/*i* datatype shrconst = 22222; *i*/
datatype shrconst = "     BBBBB"; 
/*r* struct datatype shrconst = { *r*/
  /*r* NULL, *r*/
  /*r* 22222, *r*/
  /*r* "     BBBBB" *r*/
  /*r* }; *r*/

/* pointer to shared data */
dataptr bufptr;

/* pointer to pointer to shared data */
dataptr *bufptrptr;

/* mailbox semaphore controlling access to shared memory */
ulong sema;

/* global name of mailbox semaphore */
char name[] = "Shared Memory Semaphore";


/**********************************************************************
*  main  -  check for DESQview present and enable required extensions.
***********************************************************************/

main () {
  /* initialize C interfaces and get API version number */
  version = api_init();

  /* if DESQview is not running or version is too low, display a message */
  if (version < required) {
    printf ("This program requires DESQview version %d.02%d or later.\n",
             required/256,required%256);
    }

  else {
    /* tell DESQview what extensions to enable and start application */
    api_level (required);
    program_body();

    /* disable C interfaces and return from program */
    api_exit();
    }

  }


/********************************************************************
/*  program_body
/********************************************************************/

program_body () {

  /* get object handle */
  win = win_me();

  /* find named mailbox semaphore */
  sema = mal_find (name,sizeof (name));

  /* read pointer to mailed data */
  status = mal_read (mal_me(),&bufptrptr,&mallng);

  /* assign pointer to shared data by dereferencing pointer to pointer */
  bufptr = *bufptrptr;

  /* get the application task handle of other process */
  apphana = mal_addr (mal_me());

  /* loop till handle of other process is no longer valid */
  while (api_isobj (apphana)) {

    /* lock semaphore */
    mal_lock (sema);

    /* loop REPS times */
    for (i = 1; i <= REPS; i++) {

      /* read & display current contents & address of shared data */
      /*i* win_printf (win,"%i at %Fp\n",*bufptr,bufptr); *i*/
      win_printf (win,"%s at %Fp\n",bufptr,bufptr); 
      /*r* win_printf (win,"%i %s at %Fp\n",bufptr->lng,bufptr->data,bufptr); *r*/

      /* modify contents of shared data */
      /*i* *bufptr = shrconst; *i*/
      strcpy (bufptr,shrconst);
      /*r* bufptr->lng = shrconst.lng; *r*/
      /*r* strcpy (bufptr->data,shrconst.data); *r*/
      }

    /* unlock semaphore */
    mal_unlock (sema);
    }

  }
